// =====================================================
//
//  lazmla
//
// A lot of code and many ideas stolen from:
//   miniLA_win
//
//   (c) miniLA Team
//
// =====================================================
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this package; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.


unit usb_io;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Dialogs;

const
  default_description = 'miniLA USB Interface A';


function USB_ReadReg(Addr: Integer; Immediate: Boolean): Longint;
procedure USB_WriteReg(Addr: Integer; Val: Byte; Immediate: Boolean);
procedure USB_Init;
procedure USB_Close;


implementation

uses D2XXUnit, uglobdat;



type
  TLast_Access = (none, read, write);
//  Names = Array[1..20] of string;
//  Names_Ptr = ^Names;
  DataBuff = Array[0..63] of byte;
  Data_Ptr = ^DataBuff;

var
  PortAIsOpen : boolean;
  OutIndex : integer;
  last_access : TLast_Access;




//=================================================================
procedure SendBytes(NumberOfBytes : integer);
//=================================================================
var i : integer;
begin
   i := Write_USB_Device_Buffer( NumberOfBytes);
   OutIndex := OutIndex - i;
end;


//=================================================================
procedure AddToBuffer(I:integer);
//=================================================================
begin
   FT_Out_Buffer[OutIndex]:= I AND $FF;
   inc(OutIndex);
end;



//=================================================================
function GetData: integer;
//=================================================================
var
   res 	: FT_Result;
   j 	: integer;
begin
   j := 0;
   // assume at least 1 byte
   repeat
     //if frmIO_USB.cbUSBDelay.Checked then
     sleep(ftd_latency);
     res := Get_USB_Device_QueueStatus;
     if FT_Q_Bytes > 0 then
      begin
        j := Read_USB_Device_Buffer(FT_Q_Bytes);
      end
     else
      begin
       // if frmIO_USB.cbUSBDelay.Checked then
       //    sleep(1) // delay for XP systems that lock up
       // else
       //???    sleep(0);
      end;
   until (j > 0) or (res <> FT_OK);

   if res <> FT_OK then
      Result := 0
   else
      Result := j;
end;

//=================================================================
function OpenPort(PortName : string) : boolean;
//=================================================================
var
   res : FT_Result;
   NoOfDevs,J : integer;
   Name : String;
begin
   PortAIsOpen := False;
   OpenPort := False;
   Name := '';

   // selected device connected?
   res := GetFTDeviceCount;
   if res <> Ft_OK then exit;
   NoOfDevs := FT_Device_Count;
   j := 0;
   if NoOfDevs > 0 then
     repeat

       // read description of next device
       repeat
	   res := GetFTDeviceDescription(J);
	   if (res <> Ft_OK) then  J := J + 1;
       until (res = Ft_OK) OR (J=NoOfDevs);

       if res <> Ft_OK then exit;

       Name := StrPas(@FT_Device_String_Buffer);

       J := J + 1;
     until (J = NoOfDevs) or (name = PortName);

   // open device
   if (name = PortName) then
    begin
      res := Open_USB_Device_By_Device_Description(name);
      if res <> Ft_OK then exit;
      OpenPort := true;
      res := Get_USB_Device_QueueStatus;
      if res <> Ft_OK then exit;
      PortAIsOpen := true;
    end
   else
      OpenPort := false;

end;

//=================================================================
// This should satisfy outstanding commands.
//
// We will use $AA and $AB as commands which
// are invalid so that the MPSSE block should echo these
// back to us preceded with an $FA
//=================================================================
function Sync_To_MPSSE : boolean;
var
   res : FT_Result;
   i,j : integer;
   Done : boolean;
begin
   Sync_To_MPSSE := false;
   res := Get_USB_Device_QueueStatus;
   if res <> FT_OK then exit;
   if (FT_Q_Bytes > 0) then
     i := Read_USB_Device_Buffer(FT_Q_Bytes);
   repeat
     Sleep(1);
     OutIndex := 0;
     AddToBuffer($AA); 				// bad command
     SendBytes(OutIndex);
     res := Get_USB_Device_QueueStatus;
     if FT_Q_Bytes = 0 then sleep(0);
   until (FT_Q_Bytes > 0) or (res <> FT_OK); 	// or timeout
   if res <> FT_OK then exit;
   i := Read_USB_Device_Buffer(FT_Q_Bytes);
   j := 0;
   Done := False;
   repeat
     if (FT_In_Buffer[j] = $FA) then
       begin
       if (j < (i-2)) then
	 begin
	 if (FT_In_Buffer[j+1] = $AA) then Done := true;
	 end;
       end;
     j := j + 1;
   until (j=i) or Done;
   OutIndex := 0;
   AddToBuffer($AB); 	// bad command
   SendBytes(OutIndex);
   sleep(ftd_latency + 1);
   repeat
     res := Get_USB_Device_QueueStatus;
   until (FT_Q_Bytes > 0) or (res <> FT_OK); 	// or timeout
   if res <> FT_OK then exit;
   i := Read_USB_Device_Buffer(FT_Q_Bytes);
   j := 0;
   Done := False;
   repeat
     if (FT_In_Buffer[j] = $FA) then
       begin
         if (j <= (i-2)) then
	  begin
	    if (FT_In_Buffer[j+1] = $AB) then Done := true;
	  end;
       end;
     j := j + 1;
   until (j=i) or Done;

   if Done then Sync_To_MPSSE := true;
end;

//=================================================================
function InitDev : boolean;
//=================================================================
var
  passed : boolean;
  res : FT_Result;
begin
   res := Set_USB_Device_LatencyTimer(ftd_latency);
   if res<>FT_OK then showmessage('Problem Set-Latency ');
   res := Set_USB_Device_BitMode($00,$00); // reset controller
   if res<>FT_OK then showmessage('Problem Reset ');
   res := Set_USB_Device_BitMode($00,$08); // enable Host Bus Emulation
   if res<>FT_OK then showmessage('Problem MCU-Mode ');
   passed := Sync_To_MPSSE;
   InitDev := passed;
end;

//=================================================================
procedure USB_Close;
//=================================================================
var res : FT_Result;
begin
   if PortAIsOpen then begin
     res := Close_USB_Device;
     if res=FT_Ok then
       PortAIsOpen := False
     else showmessage('Close_USB_Device failed!'); end;
end;


//=================================================================
procedure USB_Init;
//=================================================================
var
   Dname : string;
   passed : boolean;
   i:integer;
begin
//   if AutoDetect then
      DName := default_description;
//   else
//      DName := dev_description;

   if (Dname <> '') then
    begin
      passed := OpenPort(Dname);
      if passed then
       begin
	 // Device opened
         sleep(2);
	 passed := InitDev;
	 if  passed then
	  begin
	    // Initialisation passed
	  end
	 else
	  begin
	    raise Exception.Create('USB initialisation failed.');
	    USB_close;
	  end;
       end
      else
	begin
	  raise Exception.Create('USB device opening failed.');
	end;
      end
    else
     begin		// note: maybe redundant check
       raise Exception.Create('No USB device selected.');
     end;
end;



//=================================================================
function USB_ReadReg(Addr: Integer; Immediate: Boolean): longint;
//=================================================================
//var res	: byte;
begin
   if last_access <> read then
    begin
      AddToBuffer($91); 		// read extended
      AddToBuffer($1);			// Address high
    end
   else
      AddToBuffer($90);			// read short

   AddToBuffer(Addr);	 		// Address low
   if immediate then
    begin
      AddToBuffer($87); 		// Send Immediate
      SendBytes(OutIndex);		// send off the command
      result := GetData;
    end
   else
      result := 0;
   last_access := read;
end;


//=================================================================
procedure USB_WriteReg(Addr: Integer; Val: Byte; Immediate: Boolean);
//=================================================================
begin
   if last_access <> write then
    begin
      AddToBuffer($93);			// write extended
      AddToBuffer($0); 			// Address high
    end
   else
      AddToBuffer($92); 		// write short

   AddToBuffer(Addr);	 		// Address low
   AddToBuffer(Val); 			// Data
   if (immediate) then
      SendBytes(OutIndex); 		// send off the command
   last_access := write;
end;


end.

